/*
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.qpid.example;

import javax.jms.*;
import javax.naming.Context;
import javax.naming.InitialContext;
import java.util.Properties;

public class JMSReplyTo 
{

    public JMSReplyTo() 
    {
    }

    public static void main(String[] args) 
    {
        JMSReplyTo low = new JMSReplyTo();
        low.runTest();
    }

    private void runTest() 
    {
	try {
		String      tmp     = null;
		final String INITIAL_CONTEXT_FACTORY = "org.apache.qpid.jndi.PropertiesFileInitialContextFactory";

		final String CONNECTION_JNDI_NAME = "local";
		final String CONNECTION_NAME = "amqp://guest:guest@clientid/testpath?brokerlist='tcp://localhost:5672'";

		final String QUEUE_JNDI_NAME = "amq.direct";
		final String QUEUE_NAME = "goodbye.Queue";

		// Set the properties ... 
		Properties properties = new Properties();
		properties.put(Context.INITIAL_CONTEXT_FACTORY, INITIAL_CONTEXT_FACTORY);
		properties.put("connectionfactory." + CONNECTION_JNDI_NAME , CONNECTION_NAME);
		properties.put("queue." + QUEUE_JNDI_NAME , QUEUE_NAME);

		// Create the initial context
		Context context = new InitialContext(properties);
		ConnectionFactory connectionFactory = (ConnectionFactory)context.lookup(CONNECTION_JNDI_NAME);
		Connection connection = connectionFactory.createConnection();
		connection.start();

		Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
		// Queue queue = (Queue)context.lookup(QUEUE_JNDI_NAME);
		Destination dest = (Destination) context.lookup(QUEUE_JNDI_NAME);

		MessageProducer messageProducer = session.createProducer(dest);
		MessageConsumer messageConsumer = session.createConsumer(dest);

		TextMessage message = session.createTextMessage("Low world!");
		message.setJMSReplyTo(dest);
		messageProducer.send(message);

		message = (TextMessage)messageConsumer.receive();
		System.out.println(message.getJMSDestination().toString());	
		System.out.println("Queue name: " + ((javax.jms.Queue)message.getJMSDestination()).getQueueName());

		context.close();
		connection.close();
	}
	catch (Exception exp) 
	{
		exp.printStackTrace();
	}
    }
}
