/*
 *
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *
 */

package org.apache.qpid.example;

import javax.jms.*;
import javax.naming.Context;
import javax.naming.InitialContext;
import java.util.Properties;
import javax.naming.NamingEnumeration;
import javax.naming.NameClassPair;
import javax.naming.Binding;
import javax.naming.NamingException;

public class myQueueSender 
{
    public myQueueSender() 
    {
    }

    public static void main(String[] args) 
    {
        myQueueSender low = new myQueueSender();
        low.runTest();
    }

    private void runTest() 
    {
	try {
		final String INITIAL_CONTEXT_FACTORY = "org.apache.qpid.jndi.PropertiesFileInitialContextFactory";

		final String CONNECTION_JNDI_NAME = "local";
		final String CONNECTION_NAME = "amqp://guest:guest@clientid/testpath?brokerlist='tcp://localhost:5672'";

//		final String QUEUE_JNDI_NAME = "amq.direct";
//		final String QUEUE_NAME = "goodbye.Queue";
		final String QUEUE_JNDI_NAME = "queue.myQueue";
		final String QUEUE_NAME = "myQueue;{create:always}";

		// Set the properties ... 
		Properties properties = new Properties();
		properties.put(Context.INITIAL_CONTEXT_FACTORY, INITIAL_CONTEXT_FACTORY);
		properties.put("connectionfactory." + CONNECTION_JNDI_NAME , CONNECTION_NAME);
		properties.put( QUEUE_JNDI_NAME , QUEUE_NAME);

		// Create the initial context
		Context context = new InitialContext(properties);
		listContext(context,"");
		QueueConnectionFactory connFactory = (QueueConnectionFactory) context.lookup(CONNECTION_JNDI_NAME);
		// create a queue connection
            	QueueConnection queueConn = connFactory.createQueueConnection();

		// create a queue session
		QueueSession queueSession = queueConn.createQueueSession(false,Session.DUPS_OK_ACKNOWLEDGE);
		// create a queue sender
		Queue queue = (Queue) queueSession.createQueue(QUEUE_NAME);	
		//Queue queue = (Queue) context.lookup(QUEUE_JNDI_NAME);
		QueueSender queueSender = queueSession.createSender(queue);
		queueSender.setDeliveryMode(DeliveryMode.NON_PERSISTENT);
		
		QueueReceiver queueReceiver = queueSession.createReceiver(queue);

		// create a simple message to say "Hello"
		TextMessage message = queueSession.createTextMessage("Hello");
		// send the message
		queueSender.send(message);
		System.out.println("sent: " + message.getText());
		System.out.println(message.getJMSDestination().toString());
                System.out.println("Queue name: " + ((javax.jms.Queue)message.getJMSDestination()).getQueueName());
		//receive
		message = (TextMessage) queueReceiver.receive(2L);
		System.out.println("receive: " + message.getText());
		System.out.println(message.getJMSDestination().toString());
                System.out.println("Queue name: " + ((javax.jms.Queue)message.getJMSDestination()).getQueueName());
		// close the queue connection
		queueConn.close();

		context.close();
	}
	catch (Exception exp) 
	{
		exp.printStackTrace();
	}
    }

	/**
	 * * Recursively exhaust the JNDI tree
	 * */
	private void listContext(Context ctx, String indent) {
	try {
	   NamingEnumeration list = ctx.listBindings("");
	   while (list.hasMore()) {
	       Binding item = (Binding) list.next();
	       String className = item.getClassName();
	       String name = item.getName();
	       System.out.println( indent + className + " " + name);
	       Object o = item.getObject();
	       if (o instanceof javax.naming.Context) {
	    listContext((Context) o, indent + " ");
	       }
	   }
	} catch (NamingException ex) {
	   System.out.println( "JNDI failure: " + ex.toString());
	}
	}
}
